@@ -17,19 +17,23 @@
 # under the License.
 """Sphinx extension for automatically generating API documentation
 from Pecan controllers exposed through WSME.
-
 """
-import inspect
 
+import inspect
+from functools import reduce
+
+import six
 from docutils import nodes
 from docutils.parsers import rst
 from docutils.statemachine import ViewList
-
-from sphinx.util.nodes import nested_parse_with_titles
+from sphinx.util import logging
 from sphinx.util.docstrings import prepare_docstring
+from sphinx.util.nodes import nested_parse_with_titles
 
 import wsme.types
 
+LOG = logging.getLogger(__name__)
+
 
 def import_object(import_name):
     """Import the named object and return it.
@@ -53,7 +57,7 @@ def http_directive(method, path, content):
     :param content: Text describing the endpoint.
     """
     method = method.lower().strip()
-    if isinstance(content, basestring):
+    if isinstance(content, six.string_types):
         content = content.splitlines()
     yield ''
     yield '.. http:{method}:: {path}'.format(**locals())
@@ -93,18 +97,18 @@ class RESTControllerDirective(rst.Directive):
         blank_line = docstring[-1]
         docstring = docstring[:-1]  # remove blank line appended automatically
 
-
         if hasattr(method, '_wsme_definition'):
             funcdef = method._wsme_definition
 
             # Add the parameter type information. Assumes that the
             # developer has provided descriptions of the parameters.
             for arg in funcdef.arguments:
-                docstring.append(':type %s: %s' %
-                                 (arg.name, datatypename(arg.datatype)))
+                docstring.append(
+                    ':type %s: %s' % (arg.name, datatypename(arg.datatype)))
 
-            # Add a blank line before return type to avoid the formatting issues
-            # that are caused because of missing blank lines between blocks
+            # Add a blank line before return type to avoid the formatting
+            # issues that are caused because of missing blank lines between
+            # blocks
             docstring.append(blank_line)
 
             # Add the return type
@@ -119,9 +123,6 @@ class RESTControllerDirective(rst.Directive):
         return directive
 
     def make_rst_for_controller(self, path_prefix, controller):
-        env = self.state.document.settings.env
-        app = env.app
-
         controller_path = path_prefix.rstrip('/')
 
         # Some of the controllers are instantiated dynamically, so
@@ -144,25 +145,25 @@ class RESTControllerDirective(rst.Directive):
 
         lines = []
 
-        for method_name, http_method_name in [('get_all', 'get'),
-                                              ('get', 'get'),
-                                          ]:
-            app.info('Checking %s for %s method' % (controller, method_name))
+        for method_name, http_method_name in [
+            ('get_all', 'get'),
+            ('get', 'get'),
+        ]:
+            LOG.debug('Checking %s for %s method', controller, method_name)
             method = getattr(controller, method_name, None)
             if method and method.exposed:
-                app.info('Found method: %s' % method_name)
+                LOG.debug('Found method: %s', method_name)
                 lines.extend(
                     self.make_rst_for_method(
                         controller_path,
                         method,
                         http_method_name,
-                    )
-                )
+                    ))
 
         # Handle the special case for get_one(). The path should
         # include the name of the argument used to find the object.
         if hasattr(controller, 'get_one') and controller.get_one.exposed:
-            app.info('Found method: get_one')
+            LOG.debug('Found method: get_one')
             funcdef = controller.get_one._wsme_definition
             first_arg_name = funcdef.arguments[0].name
             path = controller_path + '/(' + first_arg_name + ')'
@@ -171,29 +172,28 @@ class RESTControllerDirective(rst.Directive):
                     path,
                     controller.get_one,
                     'get',
-                )
-            )
+                ))
 
-        for method_name, http_method_name in [('post', 'post'),
-                                              ('put', 'put'),
-                                              ('delete', 'delete'),
-                                              ('patch', 'patch'),
-                                          ]:
-            app.info('Checking %s for %s method' % (controller, method_name))
+        for method_name, http_method_name in [
+            ('post', 'post'),
+            ('put', 'put'),
+            ('delete', 'delete'),
+            ('patch', 'patch'),
+        ]:
+            LOG.debug('Checking %s for %s method', controller, method_name)
             method = getattr(controller, method_name, None)
             if method and method.exposed:
-                app.info('Found method: %s' % method_name)
+                LOG.debug('Found method: %s', method_name)
                 lines.extend(
                     self.make_rst_for_method(
                         controller_path,
                         method,
                         http_method_name,
-                    )
-                )
+                    ))
 
         # Look for exposed custom methods
         for name in sorted(controller._custom_actions.keys()):
-            app.info('Adding custom method: %s' % name)
+            LOG.debug('Adding custom method: %s', name)
             path = controller_path + '/' + name
             actions = controller._custom_actions[name]
             for action in actions:
@@ -204,30 +204,25 @@ class RESTControllerDirective(rst.Directive):
                     method = getattr(controller, method_name)
                 else:
                     method = getattr(controller, name)
-                app.info('Custom method %s uses action %s' % (method, action))
+                LOG.debug('Custom method %s uses action %s', method, action)
                 lines.extend(
                     self.make_rst_for_method(
                         path,
                         method,
                         action.lower(),
-                    )
-                )
+                    ))
 
         return lines
 
     def run(self):
-        env = self.state.document.settings.env
-        app = env.app
         controller_id = self.arguments[0]
-        app.info('found root-controller %s' % controller_id)
+        LOG.debug('found root-controller %s', controller_id)
 
         result = ViewList()
         controller = import_object(self.arguments[0])
 
         for line in self.make_rst_for_controller(
-                self.options.get('webprefix', '/'),
-                controller):
-            app.info('ADDING: %r' % line)
+                self.options.get('webprefix', '/'), controller):
             result.append(line, '<' + __name__ + '>')
 
         node = nodes.section()
@@ -239,5 +234,4 @@ class RESTControllerDirective(rst.Directive):
 
 
 def setup(app):
-    app.info('Initializing %s' % __name__)
     app.add_directive('rest-controller', RESTControllerDirective)
